-- ============================================================================
-- === Autogate.Lua
-- === Mod my [LSMT] Modding Team 
-- === LS25 /FS25
-- === Script by [LSMT] BaTt3RiE @ 2025
-- === Ver 1.0.0.0
-- ============================================================================

AutoGateEvent = {}
local AutoGateEvent_mt = Class(AutoGateEvent, Event)

InitEventClass(AutoGateEvent, "AutoGateEvent")

function AutoGateEvent.emptyNew()
    local self = Event.new(AutoGateEvent_mt)
    return self
end

function AutoGateEvent.new(animatedObject, open)
    local self = AutoGateEvent.emptyNew()
    self.animatedObject = animatedObject
    self.open = open
    return self
end

function AutoGateEvent:writeStream(streamId, connection)
    NetworkUtil.writeNodeObject(streamId, self.animatedObject)
    streamWriteBool(streamId, self.open)
end

function AutoGateEvent:readStream(streamId, connection)
    self.animatedObject = NetworkUtil.readNodeObject(streamId)
    self.open = streamReadBool(streamId)
    self:run(connection)
end

function AutoGateEvent:run(connection)
    if self.animatedObject ~= nil then
        AutoGate:autoGateSet(self.animatedObject, self.open, true)
    end
    if not connection:getIsServer() then
        g_server:broadcastEvent(AutoGateEvent.new(self.animatedObject, self.open), nil, connection, self.animatedObject)
    end
end

AutoGate = {}
DEBUG = false

local function debugPrint(...)
    if DEBUG then
        print(...)
    end
end

function AutoGate:autoGateSet(animatedObject, open, noEventSend)

    animatedObject.lastGateState = animatedObject.lastGateState or false
    animatedObject.lastEventTime = animatedObject.lastEventTime or 0
    local currentTime = g_currentMission.time

        if open then
            if animatedObject.animation.time < 1 then
                animatedObject.animation.direction = 1
            end
        else
            if animatedObject.animation.time > 0 then
                animatedObject.animation.direction = -1
            end
        end
        animatedObject:raiseActive()

    if not noEventSend then
        local stateChanged = open ~= animatedObject.lastGateState
        local timeToRefresh = (currentTime - animatedObject.lastEventTime) > 1000
        
        if stateChanged or timeToRefresh then
            animatedObject.lastGateState = open
            animatedObject.lastEventTime = currentTime
            
            debugPrint("AutoGate: Sende Event - Zustand: " .. tostring(open) .. 
                      ", Grund: " .. (stateChanged and "Statusänderung" or "Regelmäßiges Update"))
            
            if g_server ~= nil then
                g_server:broadcastEvent(AutoGateEvent.new(animatedObject, open), nil, nil, animatedObject)
            else
                g_client:getServerConnection():sendEvent(AutoGateEvent.new(animatedObject, open))
            end
        end
    end
end

function AutoGate:triggerCallback(animatedObject, triggerId, otherId, onEnter, onLeave, onStay, anotherID)
    if animatedObject.oldTriggerCallback then
        animatedObject.oldTriggerCallback(animatedObject, triggerId, otherId, onEnter, onLeave, onStay, anotherID)
    end

    if onEnter then
        local object = g_currentMission.nodeToObject[otherId]
        if object ~= nil and object.getOwnerFarmId ~= nil then

            local currentTime = g_currentMission.time
            animatedObject.keepActiveUntil = currentTime + 10500
            AutoGate:autoGateSet(animatedObject, true)
            animatedObject.gateCloseTimer = currentTime + 10000
        end
    end
end

function AutoGate:update(animatedObject, dt)

    if animatedObject.gateCloseTimer ~= nil or 
    (animatedObject.keepActiveUntil ~= nil and g_currentMission.time < animatedObject.keepActiveUntil) then
     animatedObject:raiseActive()
    end

    if animatedObject.gateCloseTimer ~= nil then
        local currentTime = g_currentMission.time

        local timeLeft = animatedObject.gateCloseTimer - currentTime
        if timeLeft % 1000 < 20 then
            debugPrint("AutoGate: Timer läuft - Noch " .. 
                      string.format("%.1f", timeLeft/1000) .. " Sekunden bis Schließen")
        end
        
        if currentTime > animatedObject.gateCloseTimer then
            debugPrint("AutoGate: Timer abgelaufen - Schließe Tor")
            AutoGate:autoGateSet(animatedObject, false)
            animatedObject.gateCloseTimer = nil
        end
    end
end

if AnimatedObject.triggerCallback ~= nil then
    AnimatedObject.oldTriggerCallback = AnimatedObject.triggerCallback
end
AnimatedObject.triggerCallback = Utils.overwrittenFunction(AnimatedObject.triggerCallback, function(self, triggerId, otherId, onEnter, onLeave, onStay, anotherID)
    AutoGate:triggerCallback(self, triggerId, otherId, onEnter, onLeave, onStay, anotherID)
end)

local UPDATE_INTERVAL = 1
local frameCounter = 0

local baseUpdate = AnimatedObject.update
function AnimatedObject:update(dt)
    if baseUpdate then
        baseUpdate(self, dt)
    end

    frameCounter = frameCounter + 1
    if frameCounter >= UPDATE_INTERVAL then
        frameCounter = 0
        AutoGate:update(self, dt)
    end
end